from orionis.container.contracts.facade import IFacade
from orionis.support.performance.contracts.counter import IPerformanceCounter

class PerformanceCounter(IPerformanceCounter, IFacade):
        ...
